/*___________________________________________________________________________*/
/* IBM Optimization Solutions -- Stochastic Solutions/Extensions             */
/* scn_c_api.h -        C-API header file                                    */
/* (c) IBM Corp, 1999, 2000 All rights reserved.                             */
/*                                                                           */
/* US Government Users Restricted Rights - Use, duplication, or disclosure   */
/* restricted by GAS ADP Schedule Contract with IBM Corp.                    */
/*___________________________________________________________________________*/

#ifdef __cplusplus
extern "C"{
#endif

#ifndef SCN_C_API
#define SCN_C_API


#ifndef OSLLIBAPI
#ifdef OSLMSDLL
#define OSLLINKAGE __stdcall
#define OSLLIBAPI __declspec(dllimport)
#endif
#ifdef OSLMSSTAT
#define OSLLINKAGE __stdcall
#define OSLLIBAPI
#endif
#ifdef OSLWCSTAT
#define OSLLINKAGE
#define OSLLIBAPI
#endif
#endif

#ifndef EKKSTOCH
#define EKKSTOCH
        typedef char EKKStoch;
#endif
#ifndef EKKTREE
#define EKKTREE
        typedef char EKKTree;
#endif

#ifndef DISTRIB_H
#define DISTRIB_H

        /* Problem Types */

        typedef enum {linear=1, quadratic=2} CoreType;

        /* Replace/Add core values */

#define REPLACE_CORE_VALUES 1
#define ADD_TO_CORE_VALUES  0

#define DISTN_NOT_IMPLEMENTED   0
#define SCENARIOS               1
#define INDEP_DISCRETE          2
#define BLOCK_DISCRETE          3

#endif

        /* OSL Stochastic Solutions C-API functions */
#include "ekk_c_api.h"                           /* This should be shipped with the OSLSE/OSLSTOCH products */

        OSLLIBAPI EKKStoch * OSLLINKAGE  ekks_createStoch(const char *modelname, long maxscn);
        /* ****
        ekks_createStoch creates an EKKStoch object with name as "modelname" and has
        space allocated to hold a maximum of "maxscn" number of scenarios.  The
        address of the newly created EKKStoch object is returned.

          INPUT:
      const char *modelname     name (character string) of the EKKStoch object to be created
      long maxscn               maximum number of scenarios to be accommodated in the EKKStoch object

                RETURNS:
                a pointer to an EKKStoch object
        **** */

        OSLLIBAPI EKKStoch * OSLLINKAGE  ekks_newStoch(EKKContext *env, const char *modelname, long maxscn);
        /* ****
        ekks_newStoch creates an EKKStoch object with name as "modelname" and has
        space allocated to hold a maximum of "maxscn" number of scenarios.  The
        address of the newly created EKKStoch object is returned.

          INPUT:
      EKKContext *env           a pointer to the OSL enviroment
      const char *modelname     name (character string) of the EKKStoch object to be created
      long maxscn               maximum number of scenarios to be accommodated in the EKKStoch object

                RETURNS:
                a pointer to an EKKStoch object
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_createCore(EKKStoch *stoch,
                long nstg, long *rstag, long *cstag,
                long irow, long icol, long iels,
                double *dobj, double *drlo, double *drup,
                double *dclo, double *dcup,
                long *mrow, long *mcol, double *dels, double *dqdg);
                /* ****
                ekks_createCore populates the core data of a stochastic model.  It describes to
                OSL the number of time stages in the model, stage numbers for rows and columns,
                and loads the LP matrix data (costs, bounds, elements) into the core data structure.

                  INPUT:
                  EKKStoch *stoch           pointer to the EKKStoch object (created by ekks_createStoch)
                  long nstg                 number of stages in the core model
                  long *rstag               array of stage numbers for rows
                  long *cstag               array of stage numbers for columns
                  long irow                 number of rows in core model
                  long icol                 number of columns in core model
                  long iels                 number of matrix elements in core model
                  double *dobj              array of objective entries
                  double *drlo              array of row lower bound entries
                  double *drup              array of row upper bound entries
                  double *dclo              array of column lower bound entries
                  double *dcup              array of column upper bound entries
                  long *mrow                array of matrix row indices
                  long *mcol                array of matrix column indices
                  double *dels              array of matrix entries
                  double *dqdg              array of quadratic matrix diagonal entries

                        RETURNS:
                        the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE ekks_addScenario(EKKStoch *stoch, long nscn, long ianc, long istg,
                double dprob, long itype, long nrow,long ncol,long nels,
                double *dobj, double *drlo, double *drup, double *dclo, double *dcup,
                long *mrow, long *mcol, double *dels, long replace );

                /* ****
                ekks_addScenario adds a scenario to stochastic problem, sets the scenario
                probability, the branching information, and stores the scenario data
                by node in an SPL-file.

                  Errors occur when branching information is inconsistent
                  or when dimensions are inconsistent with the core.

                        If this call is repeated for the same scenario, the scenario
                        probability is incremented by the amount of the new probability.
                        No other data is changed.

                          All scenario LP data is ADDED to the corresponding core data when
                          forming an LP.  This requires a convention for dealing with infinite
                          bounds in the core.  If the core value is infinite and the incoming
                          scenario value is nonzero, then a warning message is issued and the
                          incoming scenario value is set to zero.

                                CORE VALUE      SCENARIO VALUE     ACTION              MESSAGE    RC

                                  finite          infinite           none                none        0
                                  infinite        nonzero            scen value zeroed   warning     1

                                        The LP data is NOT kept in core memory.  (It may be retrieved for
                                        the user by a call to ekks_GetNodeData.)  The LP matrix must be in
                                        "indices" sparse format.

                                          If replace = 0:

                                                LP data should be the "difference" of the intended scenario LP data
                                                with the core LP data.

                                                  If replace = 1:

                                                        Scenario LP data should "replace" the core LP data.  In this case,
                                                        the subroutine calculates the difference for you.


                                                                INPUT:
                                                                EKKStoch *stoch           pointer to the EKKStoch object (created by ekks_createStoch)
                                                                long nscn                 scenario number (1,2,...)
                                                                long ianc                 ancestor scenario number
                                                                long istg                 branching stage number
                                                                double dprob              probability of scenario
                                                                long itype                matrix type (MUST be 1 - by indices format)
                                                                long nrow                 number of rows (must match core model)
                                                                long ncol                 number of columns (must match core model)
                                                                long nels                 number of elements in scenario matrix
                                                                double *dobj              array of objective entries
                                                                double *drlo              array of row lower bound entries
                                                                double *drup              array of row upper bound entries
                                                                double *dclo              array of column lower bound entries
                                                                double *dcup              array of column upper bound entries
                                                                long *mrow                array of matrix row indices
                                                                long *mcol                array of matrix column indices
                                                                double *dels              array of matrix entries
                                                                long replace              0=add to, 1=replace core values

                                                                  RETURNS:
                                                                  the return code (0 - successful, > 0 - unsuccessful)

        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_deleteStoch(EKKStoch *stoch);
        /* ****
        ekks_deleteStoch deletes the EKKStoch object "stoch" and frees
        up memory allocated to this object.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object (created by ekks_createStoch) to be deleted

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_deleteTree(EKKTree *tree);
        /* ****
        ekks_deleteTree deletes the EKKTree object "tree" and frees
        up memory allocated to this object.

          INPUT:
      EKKTree *tree           pointer to the EKKTree object (created by ekks_createScenarioTree) to be deleted

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_copyStoch(EKKStoch *target, EKKStoch *src);
        /* ****
        ekks_copyStoch copies the contents of the EKKStoch object pointed to by "src"
        to the EKKStoch object pointed to by "target."

          INPUT:
      EKKStoch *src             pointer to the EKKStoch object to be copied from
      EKKStoch *target          pointer to the EKKStoch object to be copied to

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_describeFullModel(EKKStoch *stoch, long loadData);
        /* ****
        ekks_describeFullModel constructs an optimization model from the full
        scenario tree (i.e., all the nodes of the tree) and describes it to OSL.
        The boolean parameter "loadData" (taking on values "0" or "1") specifies
        whether or not the model to be populated with the matrix data.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object (created by ekks_createStoch)
      long loadData             specifies whether or not to load the matrix data

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_describeSubModel(EKKStoch *stoch, long imodel, long nnodes,
                long *usrndlist, long calcprob);
                /* ****
                ekks_describeSubModel constructs an optimization model, labeled as "imodel",
                from a user-provided list of nodes (usrndlist[] of length "nnodes") and
                describes the model to OSL.
                The boolean parameter "calcprob" (taking on values "0" or "1") specifies
                whether or not the node probabilities to be recomputed.

                  INPUT:
                  EKKStoch *stoch           pointer to the EKKStoch object
                  long imodel               model number being described
                  long nnodes               number of scenario tree nodes being included in the model
                  long *usrndlist           list of nodes being included in the model
                  long calcprob             specifies whether or not to recompute the node probabilities

                        RETURNS:
                        the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekkse_decomposeCutAtStage(EKKStoch *stoch, long ipcut, long loadData);
        /* ****
        ekkse_decomposeCutAtStage decomposes the muti-stage, multi-scenario tree model
        associated with the EKKStoch object "stoch" at time stage "ipcut" into
        a number of sub-tree models.  The number of such submodels is returned. The
        boolean parameter "loadData" (taking on values "0" or "1") specifies whether
        or not the submodels to be populated with the matrix data.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long ipcut                period at which the scenario tree is cut to create subproblems
      long loadData             specifies whether or not to load subproblems with matrix data

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekkse_decomposeCutByRowSize(EKKStoch *stoch, long minNumRows, long loadData);
        /* ****
        ekkse_decomposeCutByRowSize decomposes the muti-stage, multi-scenario tree model
        associated with the EKKStoch object "stoch" into a number of sub-tree models
        with at least "minNumRows" number of rows in each submodel.  The number of such submodels is
        returned. The boolean parameter "loadData" (taking on values "0" or "1") specifies
        whether or not the submodels to be populated with the matrix data.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long minNumRows           minimum number of rows in each model
      long loadData             specifies whether or not to load subproblems with matrix data

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekkse_decomposeCutByNodeSize(EKKStoch *stoch, long maxNodes, long loadData);
        /* ****
        ekkse_decomposeCutByNodeSize decomposes the muti-stage, multi-scenario tree model
        associated with the EKKStoch object "stoch" into a number of sub-tree models
        with at most "maxNodes" number of nodes in each submodel.  The number of such submodels is
        returned. The boolean parameter "loadData" (taking on values "0" or "1") specifies
        whether or not the submodels to be populated with the matrix data.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long maxNodes             maximum number of nodes in each model
      long loadData             specifies whether or not to load subproblems with matrix data

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_eitherSimplex(EKKStoch *stoch);
        /* ****
        ekks_eitherSimplex invokes either primal or dual simplex method (based
        on the model data characteristics) to solve the base optimization
        model attached to the EKKStoch object pointed to by "stoch."

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_estimateLPSize(EKKStoch *stoch);
        /* ****
        ekks_estimateLPSize estimates the size (number of rows, columns and elements)
        of the LP that is deterministically equivalent to the stochastic program.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_end();
        /* ****
        ekks_end uninitializes the OSL environment and frees up any resources associated with it.
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_endContext(EKKContext *env);
        /* ****
        ekks_endContext uninitializes the OSL environment and frees up any resources associated with it.
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_initialize();
        /* ****
        ekks_initialize creates and initializes the entire OSL environment.
        **** */
        OSLLIBAPI EKKContext * OSLLINKAGE  ekks_initializeContext();
        /* ****
        ekks_initializeContext creates and initializes the entire OSL environment and returns a pointer to the environment.
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_loadBaseModel(EKKStoch *stoch);
        /* ****
        ekks_loadBaseModel loads matrix, costs, bounds data to the base optimization
        model attached to the EKKStoch object pointed to by "stoch."

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_markIntegers(EKKStoch *stoch);
        /* ****
        ekks_markIntegers propagates the integer variable definitions in the core
        model to the multi-scenario LP model and marks the integer variables.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the total number of integer variables in the larger model.
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_markIntegersWithStagePriorities(EKKStoch *stoch, long *prty, double *uppc, double *dnpc);
        /* ****
        ekks_markIntegers propagates the integer variable definitions in the core
        model to the multi-scenario LP model and marks the integer variables and
        associates user supplied set prioroties and pseudocosts.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long *prty                an integer array of length equal to the number of stages
      double *uppc              array of up pseudocosts
      double *dnpc              array of down pseudocosts

                RETURNS:
                the total number of integer variables in the larger model.
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_bendersLSolve(EKKStoch *stoch, long startmode);
        /* ****
        ekks_bendersLSolve solves the stochastic LP using the Benders' L-Shaped Decomposition
        algorithm.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object containing valid model and data
      long startmode            specifies starting mode (0-cold, 1-hot) for the decomposition solver

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_branchAndBound(EKKStoch *stoch, const char *matrixFile, const char *basisFile);
        /* ****
        ekks_branchAndBound solves the stochastic MIP using the branch-and-bound
        algorithm in OSL.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      const char*matrixFile     file name to store matrix info during branch-and-bound process
      const char*basisFile      file name to store basis info during branch-and-bound process

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_printNodeSolution(EKKStoch *stoch, long scenario, long stage, long printmode);
        /* ****
        ekks_printNodeSolution outputs the row/column solutions at the node (of a
        multi-scenario, multi-stage tree) specified by the scenario labeled by "scenario"
        and the stage labeled by "stage."

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long scenario             an integer labeling a scenario
      long stage                an integer labeling a stage
      long printmode            mode of printing (0-Columns, 1-Rows)

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getNodeSolution(EKKStoch *stoch, long scenario, long stage, long printmode, double *solution, long *index);
        /* ****
        ekks_getNodeSolution retrieves the row/column solutions and indices at the node (of a
        multi-scenario, multi-stage tree) specified by the scenario labeled by "scenario"
        and the stage labeled by "stage", into user supplied arrays solution[], index[].

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long scenario             an integer labeling a scenario
      long stage                an integer labeling a stage
      long printmode            mode of printing (0-Columns, 1-Rows)
      double *solution          user allocated double array
      long *index               user allocated array of integers

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getScenarioSolution(EKKStoch *stoch, long scenario, long printmode, double *solution, long *index);
        /* ****
        ekks_getScenarioSolution retrieves the row/column solutions and indices for the scenario
        specified by the scenario labeled by "scenario" into user supplied arrays solution[], index[].

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long scenario             an integer labeling a scenario
      long printmode            mode of printing (0-Columns, 1-Rows)
      double *solution          user allocated double array
      long *index               user allocated array of integers

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_printNodeDualSolution(EKKStoch *stoch, long scenario, long stage, long printmode);
        /* ****
        ekks_printNodeDualSolution outputs the dual row/column solutions at the node (of a
        multi-scenario, multi-stage tree) specified by the scenario labeled by "scenario"
        and the stage labeled by "stage."

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long scenario             an integer labeling a scenario
      long stage                an integer labeling a stage
      long printmode            mode of printing (0-Columns, 1-Rows)

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getNodeDualSolution(EKKStoch *stoch, long scenario, long stage, long printmode, double *solution, long *index);
        /* ****
        ekks_getNodeSolution retrieves the dual row/column solutions and indices at the node (of a
        multi-scenario, multi-stage tree) specified by the scenario labeled by "scenario"
        and the stage labeled by "stage", into user supplied arrays solution[], index[].

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long scenario             an integer labeling a scenario
      long stage                an integer labeling a stage
      long printmode            mode of printing (0-Columns, 1-Rows)
      double *solution          user allocated double array
      long *index               user allocated array of integers

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getScenarioDualSolution(EKKStoch *stoch, long scenario, long printmode, double *solution, long *index);
        /* ****
        ekks_getScenarioSolution retrieves the dual row/column solutions and indices for the scenario
        specified by the scenario labeled by "scenario" into user supplied arrays solution[], index[].

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long scenario             an integer labeling a scenario
      long printmode            mode of printing (0-Columns, 1-Rows)
      double *solution          user allocated double array
      long *index               user allocated array of integers

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_printObjectiveDistribution(EKKStoch *stoch);
        /* ****
        ekks_printObjectiveDistribution outputs the distribution of scenario objective function
        values (at the optimal solution) against the scenario numbers.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_messagePrintOff(EKKStoch *stoch, long extmsgnum);
        /* ****
        ekks_messagePrintOff turns off the printing of the OSL message text associated
        with the message number "extmsgnum."

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long extmsgnum            an integer describing an OSL message number

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_messagePrintOn(EKKStoch *stoch, long extmsgnum);
        /* ****
        ekks_messagePrintOff turns on the printing of the OSL message text associated
        with the message number "extmsgnum."

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long extmsgnum            an integer describing an OSL message number

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_readNativeData(EKKStoch *stoch, char *splfilename);
        /* ****
        ekks_readNativeData reads the stochastic data from a file that is in the OSL native format.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      char *splfilename         name of the file to read the stochastic data (in the OSL native format) from

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_setCoreProblemType(EKKStoch *stoch, CoreType type);
        /* ****
        ekks_setCoreProblemType sets the problem type (linear or quqdratic) for the core model
        associated with the "stoch" object.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      CoreType type             a binary indicator (linear, quadratic) specifying the problem type for the core model data

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekkse_solveParametricModel(EKKStoch *stoch,
                const double *parobj, const double *parrowlower, const double * parrowupper,
                const double *parcollower, const double * parcolupper, long mask,
                long (*report) (EKKModel *) );
                /* ****
                ekkse_solveParametricModel solves the parametric full (detrministic) model. If the real control variable Rslambda
                is non-zero, then Rslambda times the perturbation vector is added to the
                original and the resulting problem is solved with the simplex method.
                Thereafter, the perturbation is added in increments multiplied by
                Rslambdadelta up to the maximum Rslambdalim.

                  It is more efficient to set up the problem so that the original
                  is solved with base OSL's ekk_primalSimplex(), then call ekks_solveParametricModel() with lambda=0.

                        Each time the incremented problem is solved, the function pointed to
                        by report() is called with the OSL EKKModel attached to stoch as
                        argument.

                          The mask argument is used as:
                          if (mask > 15) -- perform parametric analysis on the Column Upper Bounds Vector
                          if (mask > 7)  -- perform parametric analysis on the Column Lower Bounds Vector
                          if (mask > 3)  -- perform parametric analysis on the Row Upper Bounds Vector
                          if (mask > 1)  -- perform parametric analysis on the Row Lower Bounds Vector
                          if (mask > 0)  -- perform parametric analysis on the Objective Function Vector

                                INPUT:
                                EKKStoch *stoch            model pointer initialized by ekks_describeFullModel()
                                const double *parobj       a pointer to the Objective Function Vector
                                const double *parrowlower  a pointer to the Row Lower Bounds Vector
                                const double *parrowupper  a pointer to the Row Upper Bounds Vector
                                const double *parcollower  a pointer to the Column Lower Bounds Vector
                                const double *parcolupper  a pointer to the Column Upper Bounds Vector
                                long mask                  a bit mask specifying the type(s) of parametric analysis to perform
                                long (*report(EKKModel *)) a pointer to the function to be called after solution

                                  RETURNS:
                                  the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_solveParametricCoreModel(EKKStoch *stoch,
                const double *parobj, const double *parrowlower, const double * parrowupper,
                const double *parcollower, const double * parcolupper, long mask,
                long (*report) (EKKModel *) );
                /* ****
                ekks_solveParametricCoreModel solves the parametric core model. If the real control variable Rslambda
                is non-zero, then Rslambda times the perturbation vector is added to the
                original and the resulting problem is solved with the simplex method.
                Thereafter, the perturbation is added in increments multiplied by
                Rslambdadelta up to the maximum Rslambdalim.

                  It is more efficient to set up the problem so that the original
                  is solved with base OSL's ekk_primalSimplex(), then call ekks_solveParametricModel() with lambda=0.

                        Each time the incremented problem is solved, the function pointed to
                        by report() is called with the OSL EKKModel attached to stoch as
                        argument.

                          The mask argument is used as:
                          if (mask > 15) -- perform parametric analysis on the Column Upper Bounds Vector
                          if (mask > 7)  -- perform parametric analysis on the Column Lower Bounds Vector
                          if (mask > 3)  -- perform parametric analysis on the Row Upper Bounds Vector
                          if (mask > 1)  -- perform parametric analysis on the Row Lower Bounds Vector
                          if (mask > 0)  -- perform parametric analysis on the Objective Function Vector

                                INPUT:
                                EKKStoch *stoch            model pointer initialized by ekks_describeFullModel()
                                const double *parobj       a pointer to the Objective Function Vector
                                const double *parrowlower  a pointer to the Row Lower Bounds Vector
                                const double *parrowupper  a pointer to the Row Upper Bounds Vector
                                const double *parcollower  a pointer to the Column Lower Bounds Vector
                                const double *parcolupper  a pointer to the Column Upper Bounds Vector
                                long mask                  a bit mask specifying the type(s) of parametric analysis to perform
                                long (*report(EKKModel *)) a pointer to the function to be called after solution

                                  RETURNS:
                                  the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_setIntegersAtCore(EKKStoch *stoch, long numints, long *intnums, long *intType);
        /* ****
        ekks_setIntegersAtCore marks a set of columns in the core model as integers.  The column numbers for the desired
        set are supplied by the user in an integer array, intnums[numints].  The integer array intType[numints]
        specifies the types of each integer (1- binary, 2-general integer, 3-SOS2, 0- continuous).

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long numints              number of integers to describe
      long *intnums             an array of integers describing column numbers of the core model
      long *intType             an array of integers specifying the types of each integer column

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_writeNativeData(EKKStoch *stoch, char *splfilename);
        /* ****
        ekks_writeNativeData writes the stochastic data using the OSL native format to a file.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      char *splfilename         name of the file to write the stochastic data (in the OSL native format) to

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_generateSamples(EKKStoch *stoch, long numsamples, long samplemode);
        /* ****
        ekks_generateSamples draws "numsamples" number of samples from the distribution underlying scenario tree data.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long numsamples           number of samples to draw from the underlying distribution
      long samplemode           specifies the sampling mode
          =0 - uniform random sampling (this is the only supported mode for now)

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_readSMPSData(EKKStoch *stoch, const char *corefile, const char *timefile, const char *stochfile);  /* return type of distn: SCENARIOS, INDEP_DISCRETE, BLOCK_DISCRETE */

           /* ****

           ekks_readSMPSData reads the stochastic data (in the SMPS format) from 3 files -- a "core" file containing the core model data,

           a "time" file contatining the time stage structure information, and a "stoch" file containing

           the information on the random elements of the core model.



                 INPUT:

                 EKKStoch *stoch           pointer to the EKKStoch object

                 char *corefile            name of the "core" file

                 char *timefile            name of the "time" file

                 char *stochfile           name of the "stoch" file



                   RETURNS:

                   the type (1=SCENARIOS, 2=INDEP_DISCRETE, 3=BLOCK_DISCRETE) of distribution data in the "stoch" file
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_outMatrixSMPS(EKKStoch *stoch, long smpstype, long replace, const char *corefile, const char *timefile, const char *stochfile);
        /* ****
        ekks_outMatrixSMPS writes the stochastic data (in the SMPS format) to 3 files -- a "core" file containing the core model data,
        a "time" file contatining the time stage structure information, and a "stoch" file containing
        the information on the random elements of the core model.  "smpstype" is an indicator (1,2,3)
        that identifies the format type (SCENARIOS, BLOCK_DISCRETE, INDEP_DISCRETE) of the SMPS
        data to be written into the "stoch" file.  "replace" is a binary indicator that
        specifies whether (1) or not (0), the realizable values of random variables are to be written
        into the "stoch" file.  If the value of "replace" is 1, "REPLACE" will be written
        into the SCENARIOS record, and realizable values will be written.  If the value
        of "replace" is 0, "ADD" will be written into the SCENARIOS record, and increments
        over the corresponding core entry values will be written.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long smpstype             the format in which the random data to be written into the "stoch" file
      long replace              specifies whether (1) or not (0) the data in the "stoch" file is absolute data or incremental data
      char *corefile            name of the "core" file
      char *timefile            name of the "time" file
      char *stochfile           name of the "stoch" file

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_outMatrixMPS(EKKStoch *stoch, const char *mpsfile);
        /* ****
        ekks_outMatrixMPS writes the deterministic equivalent of the underlying stochastic LP
        into an MPS file.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      char *mpsfile             name of the file to write the MPS data to

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getNsubmodels(EKKStoch *stoch);
        /* ****
        ekks_getNsubmodels returns the number of decomposed submodels created by a prior user-call to
        either ekkse_decomposeCutAtStage, ekkse_decomposeCutByRowSize or ekkse_decomposeCutByNodeSize.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the number of submodels
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getCutPeriod(EKKStoch *stoch);
        /* ****
        ekks_getCutPeriod returns the current value of the time stage at which the scenario
        tree is to be cut for decomposition purposes.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the time stage at which the scenario tree is to be cut
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setCutPeriod(EKKStoch *stoch, long period);
        /* ****
        ekks_setCutPeriod sets the time stage at which the scenario tree
        is to be cut for decomposition using the method ekkse_decomposeCutAtStage.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long period               a nonnegative integer specifying the time stage
          at which the scenario tree is to be cut

                RETURNS:
                void
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_getMinNumRows(EKKStoch *stoch);
        /* ****
        ekks_getMinNumRows returns the current setting for the minimum number of rows by
        which tree is to be cut for decomposition purposes.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the minimum number of rows that each decomposed subproblem must have
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setMinNumRows(EKKStoch *stoch, long numrows);
        /* ****
        ekks_getMinNumRows sets minimum number of rows by which the scenario tree
        is to be cut for decomposition using ekkse_decomposeCutByRowSize.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long numrows              a positive number specifying the minimum number
          of rows that each decomposed subproblem must have

                RETURNS:
                void
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_getMaxsubmodels(EKKStoch *stoch);
        /* ****
        ekks_getMaxsubmodels returns the current setting for the maximum number of submodels
        that can be created by decomposition methods.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the maximum number of submodels that the scenario tree can be decomposed into
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setMaxsubmodels(EKKStoch *stoch, long submodels);
        /* ****
        ekks_setMaxsubmodels sets the maximum number of submodels that the scenario
        tree can be decomposed into.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long submodels            a positive integer specifying the maximum number
          of submodels that the scenario tree can be decomposed into

                RETURNS:
                void
        **** */
        OSLLIBAPI long OSLLINKAGE  ekks_getMaxiter(EKKStoch *stoch);
        /* ****
        ekks_getMaxiter returns the current setting for the maximum number of iterations
        (of Master and Subproblem solutions) to be performed by the decomposition crash solvers.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the maximum number of iterations to be performed by the solvers
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setMaxiter(EKKStoch *stoch, long maxiter);
        /* ****
        ekks_setMaxiter sets the maximum number of iterations (of Master and Subproblem solutions)
        to be performed by the decomposition crash solvers.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long maxiter              a positive integer specifying the maximum number
          of iterations to be performed by the solvers

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setScaleOn(EKKStoch *stoch);
        /* ****
        ekks_setScaleOn enables scaling of constraint matrix elements associated with
        the original stochastic model as well as its decomposed submodels.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setScaleOff(EKKStoch *stoch);
        /* ****
        ekks_setScaleOff disables scaling of constraint matrix elements associated with
        the original stochastic model as well as its decomposed submodels.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setPresolve(EKKStoch *stoch,long presolvetype);
        /* ****
        ekks_setPresolve enables the use of the OSL presolve function with the option
        specified by the value of presolvetype.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long presolvetype         is an indicator ( 0, 1, 2, 3 ) for the type of
          presolve to be performed. presolvetype corresponds
          to the type parameter in the calling sequence
          of the OSL presolve function.

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setPresolveOff(EKKStoch *stoch);
        /* ****
        ekks_setPresolveOff disables the use of the OSL presolve function.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setCrash(EKKStoch *stoch, long crashmode);
        /* ****
        ekks_setCrash enables the use of the OSL crash function with the type option
        specified by the value of crashmode.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long crashmode            is the mode to be used for the crash procedure. This parameter
          corresponds to the type parameter of the OSL ekkcrash function.

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setCrashOff(EKKStoch *stoch);
        /* ****
        ekks_setCrashOff disables the use of the OSL crash function.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setSimplexAlg(EKKStoch *stoch, long alg);
        /* ****
        ekks_setSimplexAlg sets the value of the OSL simplex solver equal to the value of alg.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long alg                  is an indicator ( 0, 1 or 2 )
          for the simplex algorithm to be used: 1 => primal,
          2 => dual, 0 => solver selected, on the basis of problem data.

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setLargeBoundsOn(EKKStoch *stoch);
        /* ****
        ekks_setLargeBoundsOn enables the use of large numbers for handling infinite bounds.
        This may prevent the decomposition solvers from running into numerical instabilities.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setLargeBoundsOff(EKKStoch *stoch);
        /* ****
        ekks_setLargeBoundsOff disables the use of large numbers for handling infinite bounds.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setMinimize(EKKStoch *stoch);
        /* ****
        ekks_setMinimize sets the value of the real control variable Rmaxmin to +1, which will cause any of the OSL solvers to treat the problem as a minimization.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setMaximize(EKKStoch *stoch);
        /* ****
        ekks_setMaximize sets the value of the real control variable Rmaxmin to -1, which will cause any of the OSL solvers to treat the problem as a maximization.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */
        OSLLIBAPI double OSLLINKAGE  ekks_getRelOptimalityGap(EKKStoch *stoch);
        /* ****
        ekks_getRelOptimalityGap returns the current setting for the target relative optimality gap.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the target relative optimality gap
        **** */
        OSLLIBAPI void OSLLINKAGE  ekks_setRelOptimalityGap(EKKStoch *stoch, double tol);
        /* ****
        ekks_setRelOptimalityGap sets the current setting for the target relative optimality gap.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      double tol                a double value specifying the target relative optimality gap

                RETURNS:
                void
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getStatus(EKKStoch *stoch);
        /* ****
        ekks_getStatus returns the current solution status.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the current solution status (0 - optimal, 1- infeasible, 2 - unbounded, >=3 - incomplete)
        **** */


        OSLLIBAPI double OSLLINKAGE  ekks_getRobjvalue(EKKStoch *stoch);
        /* ****
        ekks_getRobjvalue returns the current objective function value.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the current objective function value
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getNumScenarios(EKKStoch *stoch);
        /* ****
        ekks_getNumScenarios returns the number of scenarios contained in the tree
        associated with the stochastic model.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the current number of scenarios
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getNumStages(EKKStoch *stoch);
        /* ****
        ekks_getNumStages returns the number of time stages contained in the tree
        associated with the stochastic model.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the current number of stages
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getNumNodes(EKKStoch *stoch);
        /* ****
        ekks_getNumNodes returns the number of nodes contained in the tree
        associated with the stochastic model.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the current number of nodes
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getCoreNumcols(EKKStoch *stoch);
        /* ****
        ekks_getCoreNumcols returns the number of columns in the core model
        associated with the stochastic model.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the current number of columns in the core model
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getCoreNumrows(EKKStoch *stoch);
        /* ****
        ekks_getCoreNumrows returns the number of rows in the core model
        associated with the stochastic model.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                the current number of rows in the core model
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getNumcolsAtStage(EKKStoch *stoch, long stg);
        /* ****
        ekks_getNumcolsAtStage returns the number of columns associated with the stage
        label "stg" in the core model.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long stg                  stage number identifier (a positive label)

                RETURNS:
                the current number of columns at this stage in the core model
        **** */

        OSLLIBAPI long OSLLINKAGE  ekks_getNumrowsAtStage(EKKStoch *stoch, long stg);
        /* ****
        ekks_getNumrowsAtStage returns the number of rows associated with the stage
        label "stg" in the core model.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long stg                  stage number identifier (a positive label)

                RETURNS:
                the current number of rows at this stage in the core model
        **** */

        OSLLIBAPI EKKTree * OSLLINKAGE ekks_createScenarioTree(long *arr, long narr, long mxsmp, double dp);

        /* ****
        ekks_createScenarioTree creates tree and initializes structures.
        Creates first scenario from data values in arr[].

          INPUT:
          long *arr[]          integer array of data values
          long narr            number of data values (stages) in arr[]
          long mxsmp           maximum number of samples to be stored
          long dp              probability of events

                RETURNS:
                EKKTree *tree        pointer to EKKTree structure
        **** */

        OSLLIBAPI long OSLLINKAGE ekks_addEventToTree(EKKTree *tree, long *arr, long narr, double dp);

        /* ****
        ekks_addEventToTree updates scenario tree with data values in arr[].
        If arr[] is already in tree, then it increments counters
        and returns the original scenario number,
        otherwise it creates new scenario from data values in arr[]
        and returns the newly created scenario number.

          INPUT:
          EKKTree *tree    pointer to tree initialize by ekks_InitializeTree.
          long *arr[]      integer array of data values
          long narr        number of data values (stages) in arr[]
          double dp        probability of events


                  RETURNS:
                  scenario number as described above, if successful; otherwise
                  returns:          -1 if maximum number of samples exceeded
                  -2 if tree not initialized or corrupted

        **** */

        OSLLIBAPI long OSLLINKAGE ekks_getScenarioFromTree(EKKTree *tree,long nsmp, long *ianc,
                long *istg, double *wgt, long *nscn, long **arr);
                /* ****

                  Finds scenario information for sample number nsmp.

                        NB: if branching stage, "istg", has value 1, this means
                        that the scenario branches from the root (ianc=0).

                          INPUT:
                          EKKTree *tree  pointer to tree initialized by ekks_createScenarioTree.
                          long nsmp      sample number (first sample is 0, etc)

                                OUTPUT:
                                long *ianc     pointer to ancestor scenario number.
                                long *istg     pointer to branching stage number
                                double *wgt    scenario weight
                                long *nscn     scenario number
                                long **arr       pointer to data array for scenario

                                  RETURNS:
                                  return code:      0 if successful
                                  1 if maximum number of samples exceeded
                                  2 if tree not initialized or corrupted

        **** */

        /* OSL Stochastic Extensions C-API additional functions */

        OSLLIBAPI long OSLLINKAGE  ekkse_copyCurrentModel(EKKStoch *stoch, EKKModel *model);
        /* ****
        ekkse_copyCurrentModel copies the contents of the current EKKModel object associated
        with "stoch" to the target EKKModel object pointed to by "model."

          INPUT:
      EKKStoch *stoch           pointer to the source EKKStoch object
      EKKModel *model           pointer to the target EKKModel object

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI EKKStoch ** OSLLINKAGE  ekks_getSubmodels(EKKStoch *stoch);
        /* ****
        ekks_getSubmodels returns a pointer to an array of EKKStoch objects created
        by one of the decomposition methods called by the user. If no decomposition
        has been performed, NULL is returned.

          INPUT:
      EKKStoch *stoch           pointer to the source EKKStoch object

                RETURNS:
                a pointer to an array of EKKStoch objects
        **** */

        OSLLIBAPI EKKModel * OSLLINKAGE  ekkse_getBaseModel(EKKStoch *stoch);
        /* ****
        ekkse_getBaseModel returns a pointer to the base EKKModel object associated
        with "stoch."  The distinction between the "base model" and the "current model" is
        that the base model is a skeleton model with no data, and has a set of default parameter
        settings, whereas, the current model is a snapshot of the model at any given time, after
        the model has been manipulated by the user.

          INPUT:
      EKKStoch *stoch           pointer to the source EKKStoch object

                RETURNS:
                a pointer to an EKKModel object
        **** */

        OSLLIBAPI EKKModel * OSLLINKAGE  ekkse_getCurrentModel(EKKStoch *stoch);
        /* ****
        ekkse_getCurrentModel returns a pointer to the current EKKModel object associated
        with "stoch."  The distinction between the "base model" and the "current model" is
        that the base model is a skeleton model with no data, and has a set of default parameter
        settings, whereas, the current model is a snapshot of the model at any given time, after
        the model has been manipulated by the user.

          INPUT:
      EKKStoch *stoch           pointer to the source EKKStoch object

                RETURNS:
                a pointer to an EKKModel object
        **** */
        OSLLIBAPI long OSLLINKAGE  ekkse_nestedLSolve(EKKStoch *stoch, long cutstrategy, long startmode);
        /* ****
        ekkse_nestedLSolve solves the stochastic LP using the Nested L-Shaped Decomposition
        algorithm in OSL.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object
      long cutstrategy          specifies how to cut the scenario tree to create subproblems
          =1 -- cut at stage
          =2 -- cut by row size
          =3 -- cut by node size
      long startmode            specifies starting mode (0-cold, 1-hot) for the decomposition solver

                RETURNS:
                the return code (0 - successful, > 0 - unsuccessful)
        **** */

        OSLLIBAPI void OSLLINKAGE  ekks_setFinalSimplexSolverOn(EKKStoch *stoch);
        /* ****
        ekks_setFinalSimplexSolverOn enables the use of the OSL simplex solver function to
        hotstart with the advanced solution found by ekks_bendersLSolve() or ekkse_nestedLSolve()
        methods.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */

        OSLLIBAPI void OSLLINKAGE  ekks_setFinalSimplexSolverOff(EKKStoch *stoch);
        /* ****
        ekks_setFinalSimplexSolverOn disables the use of the OSL simplex solver function to
        hotstart with the advanced solution found by ekks_bendersLSolve() or ekkse_nestedLSolve()
        methods.

          INPUT:
      EKKStoch *stoch           pointer to the EKKStoch object

                RETURNS:
                void
        **** */

        OSLLIBAPI void OSLLINKAGE  ekkse_setParallelOn(EKKStoch *stoch);
        OSLLIBAPI void OSLLINKAGE  ekkse_setParallelOnAfterVM(EKKStoch *stoch);
        OSLLIBAPI void OSLLINKAGE  ekkse_setParallelOff(EKKStoch *stoch);

#endif

#ifdef __cplusplus
}
#endif
